'use strict';

var vocabs = [
        ['rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>', 'rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>'],
        ['rdfs: <http://www.w3.org/2000/01/rdf-schema#>', 'rdfs: <http://www.w3.org/2000/01/rdf-schema#>'], 
        ['owl: <http://www.w3.org/2002/07/owl#>', 'owl: <http://www.w3.org/2002/07/owl#>'], 
        ['xsd: <http://www.w3.org/2001/XMLSchema#>', 'xsd: <http://www.w3.org/2001/XMLSchema#>'],
        ['foaf: <http://xmlns.com/foaf/0.1/>', 'foaf: <http://xmlns.com/foaf/0.1/>'],
        ['dbpedia: <http://dbpedia.org/ontology/>', 'dbpedia: <http://dbpedia.org/ontology/>'],
        ['dbpedia: <http://dbpedia.org/resource/>', 'dbpedia: <http://dbpedia.org/resource/>']
       ];

var mappingColour = '#009999';
var vocabColour = 62;
var subjectColour = '#5fb3ce';
var predicateColour = '#8FBC8F';
var predicateObjectColour = '#5FBEE1';
var linkingColour = '#009999';
var objectColour = '#7eaabc';
var graphColour = 320;
var functionColour = '#DEB887';
var parametersColour = '#CC6600';
var conceptColour = '#7eaabc';

Blockly.HSV_SATURATION = 0.55;
Blockly.HSV_VALUE = 0.65;

Blockly.Blocks['mapping'] = {
    init: function() {
    this.appendStatementInput('vocabs')
        .appendField('Mapping between')
        .setCheck(['prefix', 'predefinedprefix'])
        .setAlign(Blockly.ALIGN_RIGHT);
    this.appendStatementInput('mappings')
        .appendField('')
        .setCheck(['simplemapping', 'complexmapping'])
        .setAlign(Blockly.ALIGN_RIGHT);
    this.setColour('#005580');
    this.setPreviousStatement(true, ['mapping']);
    this.setNextStatement(true, ['mapping']);
    this.setTooltip('Creates a mapping.');
    this.contextMenu = false;
  }
};

Blockly.Blocks['prefix'] = {
    init: function() {
    this.setColour(vocabColour);
    this.appendDummyInput('prefix')
        .appendField('')
        .appendField(new Blockly.FieldTextInput('insert prefix here'), 'PREFIX');
    this.appendDummyInput('uri')
        .appendField(new Blockly.FieldLabel(': <'))
        .appendField(new Blockly.FieldTextInput('insert uri here'), 'URI')
    .appendField(new Blockly.FieldLabel('>'));
    this.setInputsInline(true);
    this.setPreviousStatement(true, ['prefix', 'predefinedprefix']);
    this.setNextStatement(true, ['prefix', 'predefinedprefix']);
    this.setTooltip('Defines a prefix.');
  }
};

Blockly.Blocks['predefinedprefix'] = {
    init: function() {
    this.setColour(vocabColour);
    this.appendDummyInput('prefix')
        .appendField('')
        .appendField(new Blockly.FieldDropdown(vocabs), 'PREFIX');
    this.setInputsInline(true);
    this.setPreviousStatement(true, ['prefix', 'predefinedprefix']);
    this.setNextStatement(true, ['prefix', 'predefinedprefix']);
    this.setTooltip('Defines a prefix.');
  }
};

Blockly.Blocks['base'] = {
    init: function() {
    this.setColour(vocabColour);
    this.appendDummyInput('base')
        .appendField('base <')
        .appendField(new Blockly.FieldTextInput('insert uri here'), 'URI')
        .appendField('>');
    this.setInputsInline(true);
    this.setPreviousStatement(true, ['mapping','prefix', 'predefinedprefix']);
    this.setNextStatement(true, ['prefix', 'predefinedprefix']);
    this.setTooltip('Defines a base uri.');
  }
};

function nextMapping() {
  var simpleMappingBlocks = getBlocksByType('simplemapping');
  var complexMappingBlocks = getBlocksByType('complexmapping');
  return (simpleMappingBlocks.length + complexMappingBlocks.length + 1);

}

Blockly.Blocks['simplemapping'] = {
    init: function() {
    this.appendDummyInput('id')
        .appendField('Id: ')
        .appendField(new Blockly.FieldTextInput(nextMapping()), 'ID');
    this.appendValueInput('source')
        .appendField('source')
        .setCheck(['concept']);
    this.appendStatementInput('properties')
        .setCheck(['property_concept']);
    this.setColour(mappingColour);
    this.setPreviousStatement(true, ['simplemapping', 'complexmapping']);//, ['correspondences']);
    this.setNextStatement(true, ['simplemapping', 'complexmapping']);//, ['correspondences']););//, 'correspondences');
    this.setTooltip('Defines a simple mapping.');
    this.setInputsInline(true);
  }, onchange: function(event) {
    if(event.type == Blockly.Events.CREATE) {
      var newBlock = workspace.getBlockById(event.blockId);
      if(newBlock == null) {
        return;
      }

      if(newBlock.type == 'simplemapping') {
        newBlock.setFieldValue((nextMapping() - 1), 'ID');
      }
    }
  }
};

Blockly.Blocks['relation_concept'] = {
  init: function() {
    this.setColour(predicateObjectColour);
    this.appendValueInput('relation')
        .appendField('relation')
        .setCheck(['relation']);
    this.appendValueInput('target')
        .appendField('target')
        .setCheck(['concept']);
    this.setPreviousStatement(true, ['properties', 'property_concept']);
    this.setInputsInline(true);
    this.setTooltip('Structure for simple mappings.');
  }
};


Blockly.Blocks['concept'] = {
    init: function() {
    this.setColour(conceptColour);
    this.appendDummyInput('concept')
        .appendField(new Blockly.FieldTextInput('concept'), 'CONCEPT');
    this.setInputsInline(true);
    this.setOutput(true, 'concept');
    this.setTooltip('Defines a concept.');
    // this.setMovable(false);
  }
};

Blockly.Blocks['relation'] = {
    init: function() {
    this.setColour(predicateColour);
    this.appendDummyInput('relation')
        .appendField(new Blockly.FieldDropdown([
                        ['class equivalent to', 'classeq'], 
                        ['property equivalent to', 'propeq'], 
                        ['individual same as', 'sameas'], 
                        ['inverse property of', 'propinv'],
                        ['subclass of', 'subclass'],
                        ['subproperty of', 'subprop'],
                        ['disjoint with', 'disjoint']
                        ]), 'RELATION');
    this.setOutput(true, 'relation');
    this.setTooltip('Defines a relation.');
  }
};

Blockly.Blocks['complexmapping'] = {
  init: function() {
    this.appendDummyInput('id')
        .appendField('Id: ')
        .appendField(new Blockly.FieldTextInput(nextMapping()), 'ID');
    this.appendStatementInput('source')
        .appendField('source')
        .setCheck(['subject']);
    this.appendStatementInput('target')
        .appendField('target')
        .setCheck(['subject']);
    this.setColour(140); 
    this.setPreviousStatement(true, ['simplemapping', 'complexmapping']);
    this.setNextStatement(true, ['simplemapping', 'complexmapping']);
    this.setTooltip('Defines a complex mapping.');
    this.setInputsInline(true);
  }, onchange: function(event) {
    if(event.type == Blockly.Events.CREATE) {
      var newBlock = workspace.getBlockById(event.blockId);
      if(newBlock == null) {
        return;
      }

      if(newBlock.type == 'complexmapping') {
        newBlock.setFieldValue((nextMapping() - 1), 'ID');
      }
    }
  }
}

Blockly.Blocks['subject'] = {
    init: function() {
    this.appendValueInput('subject')
        .appendField('subject')
        .setCheck(['variables_get', 'subjectdef']);
    this.appendStatementInput('predicate_object')
        .setCheck(['predicate_object']);
    this.setColour('#1a7fa8');
    this.setPreviousStatement(true, ['subject']);//, ['correspondences']);
    this.setNextStatement(true, ['subject']);//, ['correspondences']););//, 'correspondences');
    this.setTooltip('Defines a subject.');
    this.setInputsInline(true);
  }, onchange: function(event) {
    if(event.type == Blockly.Events.CREATE) {
      var newBlock = workspace.getBlockById(event.blockId);
      if(newBlock == null) {
        return;
      }

      if(newBlock.type == 'simplemapping') {
        newBlock.setFieldValue((nextMapping() - 1), 'ID');
      }
    }
  }
};

Blockly.Blocks['predicate_object'] = {
  init: function() {
    this.setColour(predicateObjectColour);
    this.appendValueInput('predicate')
        .appendField('predicate')
        .setCheck(['predicate', 'variables_get']);
    this.appendValueInput('object')
        .appendField('and object')
        .setCheck(['object', 'variables_get']);
    this.setPreviousStatement(true, ['predicate_object']);//, ['relations']);
    this.setNextStatement(true, ['predicate_object']);//, ['relations']);
    this.setInputsInline(true);
    this.setTooltip('Structure to define predicates and objects.');
  }
};

Blockly.Blocks['predicate'] = {
    init: function() {
    this.setColour(predicateColour);
    this.appendDummyInput('predicate')
        .appendField('using')
        // .appendField(new Blockly.FieldDropdown([['constant', 'CONSTANT'], ['template', 'TEMPLATE']]), 'type')
        .appendField(new Blockly.FieldTextInput('insert value'), 'PREDICATE');
    this.setInputsInline(true);
    this.setOutput(true, 'predicate');//, ['relation']);
    this.setTooltip('Defines a predicate.');
    // this.setMovable(false);
  }
};

Blockly.Blocks['subjectdef'] = {
    init: function() {
    this.setColour(subjectColour);
    this.appendDummyInput('subjecttype')
        .appendField('using')
        // .appendField(new Blockly.FieldDropdown([['constant', 'CONSTANT'], ['template', 'TEMPLATE']]), 'type')
    this.appendDummyInput('subjectvalue')
        // .appendField(' ')
        .appendField(new Blockly.FieldTextInput('insert value'), 'SUBJECT');
    this.setInputsInline(true);
    this.setOutput(true, ['subject', 'subjectdef']);//, ['relation']);
    this.setTooltip('Defines a subject definition.');
    // this.setMovable(false);
  }
};

Blockly.Blocks['object'] = {
    init: function() {
    this.setColour(objectColour);
    this.appendDummyInput('predicatetype')
        .appendField('using')
        .appendField(new Blockly.FieldTextInput('insert value'), 'OBJECT')
        .appendField('with options:')
        .appendField(new Blockly.FieldCheckbox('FALSE'), 'WITHOPTION');
    this.appendDummyInput('options')
        .appendField('as/with')
        .appendField(new Blockly.FieldDropdown([ ['literal', 'literal'], ['datatype', 'datatype'], ['language', 'language']]), 'OPTIONTYPE'); //['blank node', 'BNODE'],
    this.appendDummyInput('lang_type')
        .appendField(' ')
        .appendField(new Blockly.FieldTextInput('insert value'), 'lang_datatype');
   this.setInputsInline(true);
    this.setOutput(true, 'object');//, ['relation']);
    this.setTooltip('Defines an object.');
  }, onchange: function(event) {
    if(this.getFieldValue('WITHOPTION') == 'TRUE'){
      this.getInput('options').setVisible(true);
      if(this.getFieldValue('OPTIONTYPE') != 'BNODE' && this.getFieldValue('OPTIONTYPE') != 'literal'){
        this.getInput('lang_type').setVisible(true);
      } else {
        this.getInput('lang_type').setVisible(false);
      }
    } else {
      this.getInput('options').setVisible(false);
      this.getInput('lang_type').setVisible(false);
    }
    this.render();
  }
};

Blockly.Blocks['bnode'] = {
    init: function() {
    this.setColour(subjectColour);
    this.appendDummyInput('blank_node')
        .appendField('with subject as blank node');
    this.setInputsInline(true);
    this.setPreviousStatement(true, ['bnode', 'predicate_object', 'subject', 'graph']);
    this.setNextStatement(true, ['bnode', 'predicate_object', 'subject', 'graph']);
    this.setTooltip('Defines the subject as a blank node.');
  }, onchange: function(ev){
    // TODO only allowed in source
  }
};


Blockly.Blocks['variables_get_param'] = {
    init: function() {
    this.setColour(parametersColour);
    this.appendDummyInput('var')
        .appendField(new Blockly.FieldVariable('item'), 'NAME');
    this.setPreviousStatement(true);
    this.setNextStatement(true);
  }
};

Blockly.Blocks['concat'] = {
  init: function() {
    this.appendDummyInput('function')      
        .appendField('concatenating')
        .appendField('', 'PARAMS');
    this.setMutator(new Blockly.Mutator(['parameter', 'variables_get_param']));
    this.setColour(functionColour);
    this.setTooltip('Concatenation function.');
    this.arguments_ = [];
    this.as = '';
    this.setOutput(true, ['object', 'concat']);//['predicate','subject','object']
  }, 
  updateParams_: function() {
    var params = '';
    if(this.arguments_.length < 2) {
      params = 'This function must have at least 2 parameters.';
    } else {
      for (var i = 0; i < this.arguments_.length; i++) {
        var param = this.arguments_[i][1];
        if(params != '') {
            params += ', ';
        }
        params += param;
      }
    }
    Blockly.Events.disable();
    try {
      if(params != '') {
        params = params;
      }
      this.setFieldValue(params, 'PARAMS');
    } finally {
      Blockly.Events.enable();
    }
  },
  mutationToDom: function() {
    var container = document.createElement('mutation');
    for (var i = 0; i < this.arguments_.length; i++) {
      var parameter = document.createElement('arg');
      parameter.setAttribute('paramtype', this.arguments_[i][0]);
      parameter.setAttribute('paramvalue', this.arguments_[i][1]);

      container.appendChild(parameter);
    }
    
    container.setAttribute('as', this.as);

    return container;
  },
  domToMutation: function(xmlElement) {
    this.arguments_ = [];
    for (var i = 0, childNode; childNode = xmlElement.childNodes[i]; i++) {
      if (childNode.nodeName.toLowerCase() == 'arg') {
        this.arguments_.push([childNode.getAttribute('paramtype'), childNode.getAttribute('paramvalue')]);
      }
    }
    this.updateParams_();

    this.as = xmlElement.getAttribute('as');
  },
  decompose: function(workspace) {
    var containerBlock = workspace.newBlock('parameters');
    containerBlock.initSvg();
    var connection = containerBlock.getInput('STACK').connection;
    for (var i = 0; i < this.arguments_.length; i++) {
      var paramBlock = workspace.newBlock(this.arguments_[i][0]);
      paramBlock.initSvg();
      if(paramBlock.type == 'variables_get_param') {
        paramBlock.setFieldValue(this.arguments_[i][1], 'NAME');
      } else {
        paramBlock.setFieldValue(this.arguments_[i][1], 'paramvalue');
      }
      paramBlock.oldLocation = i;
      connection.connect(paramBlock.previousConnection);
      connection = paramBlock.nextConnection;
    }
    containerBlock.setFieldValue(this.as, 'AS');

    return containerBlock;
  },
  compose: function(containerBlock) {
    this.arguments_ = [];
    var paramBlock = containerBlock.getInputTargetBlock('STACK');
    while (paramBlock) {
      var parameter = '';
      if(paramBlock.type == 'variables_get_param') {
        parameter = paramBlock.getVars()[0];
      } else {
        parameter = paramBlock.getFieldValue('paramvalue');
      }
      this.arguments_.push([paramBlock.type, parameter]);
      paramBlock = paramBlock.nextConnection &&
          paramBlock.nextConnection.targetBlock();
    }
    this.updateParams_();

    this.as = containerBlock.getFieldValue('AS');
  }
};

Blockly.Blocks['strbefore'] = {
  init: function() {
    this.appendDummyInput('function')      
        .appendField('string before', 'FUNCTION_NAME')
        .appendField('', 'PARAMS');
    this.setMutator(new Blockly.Mutator(['parameter', 'variables_get_param']));
    this.setColour(functionColour);
    this.setTooltip('String before function.');
    this.arguments_ = [];
    this.as = '';
    this.setOutput(true, ['object', 'strbefore']);
  }, 
  updateParams_: function() {
    var params = '';
    var functionName = '';
    if(this.arguments_.length != 2) {
      params = 'This function must have 2 parameters.';
      functionName = 'string after';
    } else {
        params = 'string from \"' + this.arguments_[0][1] + '\" before \"' + this.arguments_[1][1] + "\"";
        functionName = '';
    }
    Blockly.Events.disable();
    try {
      if(params != '') {
        params = params;
      }
      this.setFieldValue(functionName, 'FUNCTION_NAME');
      this.setFieldValue(params, 'PARAMS');
    } finally {
      Blockly.Events.enable();
    }
  },
  mutationToDom: function() {
    var container = document.createElement('mutation');
    for (var i = 0; i < this.arguments_.length; i++) {
      var parameter = document.createElement('arg');
      parameter.setAttribute('paramtype', this.arguments_[i][0]);
      parameter.setAttribute('paramvalue', this.arguments_[i][1]);

      container.appendChild(parameter);
    }
    
    container.setAttribute('as', this.as);

    return container;
  },
  domToMutation: function(xmlElement) {
    this.arguments_ = [];
    for (var i = 0, childNode; childNode = xmlElement.childNodes[i]; i++) {
      if (childNode.nodeName.toLowerCase() == 'arg') {
        this.arguments_.push([childNode.getAttribute('paramtype'), childNode.getAttribute('paramvalue')]);
      }
    }
    this.updateParams_();

    this.as = xmlElement.getAttribute('as');
  },
  decompose: function(workspace) {
    var containerBlock = workspace.newBlock('parameters');
    containerBlock.initSvg();
    var connection = containerBlock.getInput('STACK').connection;
    for (var i = 0; i < this.arguments_.length; i++) {
      var paramBlock = workspace.newBlock(this.arguments_[i][0]);
      paramBlock.initSvg();
      if(paramBlock.type == 'variables_get_param') {
        paramBlock.setFieldValue(this.arguments_[i][1], 'NAME');
      } else {
        paramBlock.setFieldValue(this.arguments_[i][1], 'paramvalue');
      }
      paramBlock.oldLocation = i;
      connection.connect(paramBlock.previousConnection);
      connection = paramBlock.nextConnection;
    }
    containerBlock.setFieldValue(this.as, 'AS');

    return containerBlock;
  },
  compose: function(containerBlock) {
    this.arguments_ = [];
    var paramBlock = containerBlock.getInputTargetBlock('STACK');
    while (paramBlock) {
      var parameter = '';
      if(paramBlock.type == 'variables_get_param') {
        parameter = paramBlock.getVars()[0];
      } else {
        parameter = paramBlock.getFieldValue('paramvalue');
      }
      this.arguments_.push([paramBlock.type, parameter]);
      paramBlock = paramBlock.nextConnection &&
          paramBlock.nextConnection.targetBlock();
    }
    this.updateParams_();

    this.as = containerBlock.getFieldValue('AS');
  }
};

Blockly.Blocks['strafter'] = {
  init: function() {
    this.appendDummyInput('function')      
        .appendField('string after', 'FUNCTION_NAME')
        .appendField('', 'PARAMS');
    this.setMutator(new Blockly.Mutator(['parameter', 'variables_get_param']));
    this.setColour(functionColour);
    this.setTooltip('String after function.');
    this.arguments_ = [];
    this.as = '';
    this.setOutput(true, ['object', 'strafter']);
  }, 
  updateParams_: function() {
    var params = '';
    var functionName = '';
    if(this.arguments_.length != 2) {
      params = 'This function must have 2 parameters.';
      functionName = 'string after';
    } else {
        params = 'string from \"' + this.arguments_[0][1] + '\" after \"' + this.arguments_[1][1] + "\"";
        functionName = '';
    }
    Blockly.Events.disable();
    try {
      if(params != '') {
        params = params;
      }
      this.setFieldValue(functionName, 'FUNCTION_NAME');
      this.setFieldValue(params, 'PARAMS');
    } finally {
      Blockly.Events.enable();
    }
  },
  mutationToDom: function() {
    var container = document.createElement('mutation');
    for (var i = 0; i < this.arguments_.length; i++) {
      var parameter = document.createElement('arg');
      parameter.setAttribute('paramtype', this.arguments_[i][0]);
      parameter.setAttribute('paramvalue', this.arguments_[i][1]);

      container.appendChild(parameter);
    }
    
    container.setAttribute('as', this.as);

    return container;
  },
  domToMutation: function(xmlElement) {
    this.arguments_ = [];
    for (var i = 0, childNode; childNode = xmlElement.childNodes[i]; i++) {
      if (childNode.nodeName.toLowerCase() == 'arg') {
        this.arguments_.push([childNode.getAttribute('paramtype'), childNode.getAttribute('paramvalue')]);
      }
    }
    this.updateParams_();

    this.as = xmlElement.getAttribute('as');
  },
  decompose: function(workspace) {
    var containerBlock = workspace.newBlock('parameters');
    containerBlock.initSvg();
    var connection = containerBlock.getInput('STACK').connection;
    for (var i = 0; i < this.arguments_.length; i++) {
      var paramBlock = workspace.newBlock(this.arguments_[i][0]);
      paramBlock.initSvg();
      if(paramBlock.type == 'variables_get_param') {
        paramBlock.setFieldValue(this.arguments_[i][1], 'NAME');
      } else {
        paramBlock.setFieldValue(this.arguments_[i][1], 'paramvalue');
      }
      paramBlock.oldLocation = i;
      connection.connect(paramBlock.previousConnection);
      connection = paramBlock.nextConnection;
    }
    containerBlock.setFieldValue(this.as, 'AS');

    return containerBlock;
  },
  compose: function(containerBlock) {
    this.arguments_ = [];
    var paramBlock = containerBlock.getInputTargetBlock('STACK');
    while (paramBlock) {
      var parameter = '';
      if(paramBlock.type == 'variables_get_param') {
        parameter = paramBlock.getVars()[0];
      } else {
        parameter = paramBlock.getFieldValue('paramvalue');
      }
      this.arguments_.push([paramBlock.type, parameter]);
      paramBlock = paramBlock.nextConnection &&
          paramBlock.nextConnection.targetBlock();
    }
    this.updateParams_();

    this.as = containerBlock.getFieldValue('AS');
  }
};


Blockly.Blocks['minutesToSeconds'] = {
  init: function() {
    this.appendDummyInput('function')      
        .appendField('transforming seconds to minutes', 'FUNCTION_NAME')
        .appendField('', 'PARAMS');
    this.setMutator(new Blockly.Mutator(['parameter', 'variables_get_param']));
    this.setColour(functionColour);
    this.setTooltip('Transforming seconds to minutes.');
    this.arguments_ = [];
    this.as = '';
    this.setOutput(true, ['object', 'minutesToSeconds']);
  }, 
  updateParams_: function() {
    var params = '';
    var functionName = '';
    if(this.arguments_.length != 1) {
      params = 'This function must have 1 parameters.';
      functionName = 'transforming seconds to minutes';
    } else {
        params = 'transforming \"' + this.arguments_[0][1] + '\" in seconds to minutes';
        functionName = '';
    }
    Blockly.Events.disable();
    try {
      if(params != '') {
        params = params;
      }
      this.setFieldValue(functionName, 'FUNCTION_NAME');
      this.setFieldValue(params, 'PARAMS');
    } finally {
      Blockly.Events.enable();
    }
  },
  mutationToDom: function() {
    var container = document.createElement('mutation');
    for (var i = 0; i < this.arguments_.length; i++) {
      var parameter = document.createElement('arg');
      parameter.setAttribute('paramtype', this.arguments_[i][0]);
      parameter.setAttribute('paramvalue', this.arguments_[i][1]);

      container.appendChild(parameter);
    }
    
    container.setAttribute('as', this.as);

    return container;
  },
  domToMutation: function(xmlElement) {
    this.arguments_ = [];
    for (var i = 0, childNode; childNode = xmlElement.childNodes[i]; i++) {
      if (childNode.nodeName.toLowerCase() == 'arg') {
        this.arguments_.push([childNode.getAttribute('paramtype'), childNode.getAttribute('paramvalue')]);
      }
    }
    this.updateParams_();

    this.as = xmlElement.getAttribute('as');
  },
  decompose: function(workspace) {
    var containerBlock = workspace.newBlock('parameters');
    containerBlock.initSvg();
    var connection = containerBlock.getInput('STACK').connection;
    for (var i = 0; i < this.arguments_.length; i++) {
      var paramBlock = workspace.newBlock(this.arguments_[i][0]);
      paramBlock.initSvg();
      if(paramBlock.type == 'variables_get_param') {
        paramBlock.setFieldValue(this.arguments_[i][1], 'NAME');
      } else {
        paramBlock.setFieldValue(this.arguments_[i][1], 'paramvalue');
      }
      paramBlock.oldLocation = i;
      connection.connect(paramBlock.previousConnection);
      connection = paramBlock.nextConnection;
    }
    containerBlock.setFieldValue(this.as, 'AS');

    return containerBlock;
  },
  compose: function(containerBlock) {
    this.arguments_ = [];
    var paramBlock = containerBlock.getInputTargetBlock('STACK');
    while (paramBlock) {
      var parameter = '';
      if(paramBlock.type == 'variables_get_param') {
        parameter = paramBlock.getVars()[0];
      } else {
        parameter = paramBlock.getFieldValue('paramvalue');
      }
      this.arguments_.push([paramBlock.type, parameter]);
      paramBlock = paramBlock.nextConnection &&
          paramBlock.nextConnection.targetBlock();
    }
    this.updateParams_();

    this.as = containerBlock.getFieldValue('AS');
  }
};


Blockly.Blocks['percentageOf'] = {
  init: function() {
    this.appendDummyInput('function')      
        .appendField('percentage of', 'FUNCTION_NAME')
        .appendField('', 'PARAMS');
    this.setMutator(new Blockly.Mutator(['parameter', 'variables_get_param']));
    this.setColour(functionColour);
    this.setTooltip('Percentage of two values (value1/value2)*100.');
    this.arguments_ = [];
    this.as = '';
    this.setOutput(true, ['object', 'percentageOf']);
  }, 
  updateParams_: function() {
    var params = '';
    var functionName = '';
    if(this.arguments_.length != 2) {
      params = 'This function must have 2 parameters.';
      functionName = 'percentage of';
    } else {
        params = 'percentage of \"' + this.arguments_[0][1] + '\" and ' + '\"' + this.arguments_[1][1] + '\"';
        functionName = '';
    }
    Blockly.Events.disable();
    try {
      if(params != '') {
        params = params;
      }
      this.setFieldValue(functionName, 'FUNCTION_NAME');
      this.setFieldValue(params, 'PARAMS');
    } finally {
      Blockly.Events.enable();
    }
  },
  mutationToDom: function() {
    var container = document.createElement('mutation');
    for (var i = 0; i < this.arguments_.length; i++) {
      var parameter = document.createElement('arg');
      parameter.setAttribute('paramtype', this.arguments_[i][0]);
      parameter.setAttribute('paramvalue', this.arguments_[i][1]);

      container.appendChild(parameter);
    }
    
    container.setAttribute('as', this.as);

    return container;
  },
  domToMutation: function(xmlElement) {
    this.arguments_ = [];
    for (var i = 0, childNode; childNode = xmlElement.childNodes[i]; i++) {
      if (childNode.nodeName.toLowerCase() == 'arg') {
        this.arguments_.push([childNode.getAttribute('paramtype'), childNode.getAttribute('paramvalue')]);
      }
    }
    this.updateParams_();

    this.as = xmlElement.getAttribute('as');
  },
  decompose: function(workspace) {
    var containerBlock = workspace.newBlock('parameters');
    containerBlock.initSvg();
    var connection = containerBlock.getInput('STACK').connection;
    for (var i = 0; i < this.arguments_.length; i++) {
      var paramBlock = workspace.newBlock(this.arguments_[i][0]);
      paramBlock.initSvg();
      if(paramBlock.type == 'variables_get_param') {
        paramBlock.setFieldValue(this.arguments_[i][1], 'NAME');
      } else {
        paramBlock.setFieldValue(this.arguments_[i][1], 'paramvalue');
      }
      paramBlock.oldLocation = i;
      connection.connect(paramBlock.previousConnection);
      connection = paramBlock.nextConnection;
    }
    containerBlock.setFieldValue(this.as, 'AS');

    return containerBlock;
  },
  compose: function(containerBlock) {
    this.arguments_ = [];
    var paramBlock = containerBlock.getInputTargetBlock('STACK');
    while (paramBlock) {
      var parameter = '';
      if(paramBlock.type == 'variables_get_param') {
        parameter = paramBlock.getVars()[0];
      } else {
        parameter = paramBlock.getFieldValue('paramvalue');
      }
      this.arguments_.push([paramBlock.type, parameter]);
      paramBlock = paramBlock.nextConnection &&
          paramBlock.nextConnection.targetBlock();
    }
    this.updateParams_();

    this.as = containerBlock.getFieldValue('AS');
  }
};

Blockly.Blocks['parameters'] = {
  init: function() {
    this.appendDummyInput()
        .appendField('parameters');
    this.appendStatementInput('STACK');
    this.appendDummyInput('termtype')
        .appendField('as')
        .appendField(new Blockly.FieldDropdown([ ['optional', ''], ['iri', 'iri'], ['xsd:float', 'float'] ]), 'AS') // add a dynamic one depending of the function being called
        .appendField(' ');
    this.setColour(parametersColour);
    this.setTooltip('');
    this.contextMenu = false;
    this.setInputsInline(true);
  }
};

Blockly.Blocks['parameter'] = {
  init: function() {
    this.setColour(parametersColour);
    this.appendDummyInput('param')
                .appendField(new Blockly.FieldTextInput('insert value'), 'paramvalue');
    this.setPreviousStatement(true);
    this.setNextStatement(true);
    this.setTooltip('Defines a parameter.');
    this.contextMenu = false;
  }
};

function getBlocksByType(type) {
  var blocks = [];
  for (var blockID in workspace.blockDB_) {
    if (type == workspace.blockDB_[blockID].type) {
      blocks.push(workspace.blockDB_[blockID]);
    }
  }
  return(blocks);
}

Blockly.Blocks['classes'] = {
  init: function() {
    this.appendDummyInput()
        .appendField('with classes')
        .appendField('', 'PARAMS');
    this.setMutator(new Blockly.Mutator(['class_each']));
    this.setPreviousStatement(true, ['bnode', 'predicate_object', 'subject']);
    this.setNextStatement(true, ['bnode', 'predicate_object', 'subject']);
    this.setColour(subjectColour);
    this.setTooltip('Defines classes.');
    this.arguments_ = [];
  },
  setStatements_: function() {}, 
  updateParams_: function() {
    var badArg = false;
    var hash = {};
    for (var i = 0; i < this.arguments_.length; i++) {
      if (hash['arg_' + this.arguments_[i].toLowerCase()]) {
        badArg = true;
        break;
      }
      hash['arg_' + this.arguments_[i].toLowerCase()] = true;
    }
    if (badArg) {
      this.setWarningText('This construct has duplicate classes.');
    } else {
      this.setWarningText(null);
    }
    var paramString = '';
    if (this.arguments_.length) {
      paramString = this.arguments_.join(', ');
    }
    Blockly.Events.disable();
    try {
      this.setFieldValue(paramString, 'PARAMS');
    } finally {
      Blockly.Events.enable();
    }
  },
  mutationToDom: function() {
    var container = document.createElement('mutation');
    for (var i = 0; i < this.arguments_.length; i++) {
      var parameter = document.createElement('arg');
      parameter.setAttribute('name', this.arguments_[i]);
      container.appendChild(parameter);
    }
    return container;
  },
  domToMutation: function(xmlElement) {
    this.arguments_ = [];
    for (var i = 0, childNode; childNode = xmlElement.childNodes[i]; i++) {
      if (childNode.nodeName.toLowerCase() == 'arg') {
        this.arguments_.push(childNode.getAttribute('name'));
      }
    }
    this.updateParams_();
  },
  decompose: function(workspace) {
    var containerBlock = workspace.newBlock('classes_list');
    containerBlock.initSvg();
    var connection = containerBlock.getInput('STACK').connection;
    for (var i = 0; i < this.arguments_.length; i++) {
      var paramBlock = workspace.newBlock('class_each');
      paramBlock.initSvg();
      paramBlock.setFieldValue(this.arguments_[i], 'NAME');
      paramBlock.oldLocation = i;
      connection.connect(paramBlock.previousConnection);
      connection = paramBlock.nextConnection;
    }
    return containerBlock;
  },
  compose: function(containerBlock) {
    this.arguments_ = [];
    var paramBlock = containerBlock.getInputTargetBlock('STACK');
    while (paramBlock) {
      this.arguments_.push(paramBlock.getFieldValue('NAME'));
      paramBlock = paramBlock.nextConnection &&
          paramBlock.nextConnection.targetBlock();
    }
    this.updateParams_();
  }
};

Blockly.Blocks['classes_list'] = {
  init: function() {
    this.appendDummyInput()
        .appendField('classes');
    this.appendStatementInput('STACK');
    this.setColour(subjectColour);
    this.setTooltip('');
    this.contextMenu = false;
  }
};

Blockly.Blocks['class_each'] = {
  init: function() {
    this.appendDummyInput()
        .appendField('class')
        .appendField(new Blockly.FieldTextInput('class'), 'NAME');
    this.setPreviousStatement(true);
    this.setNextStatement(true);
    this.setColour(subjectColour);
    this.setTooltip('');
    this.contextMenu = false;
  }
};

Blockly.Blocks['variables_set'] = null; // Removes set variable from toolbox
Blockly.Blocks['math_change'] = null; // Removes math change from toolbox


 